import json
import math

from api.resources import Resource

from masonite_api_pagination.resources.FiltersHelper import FiltersHelper


class PaginatedResource(Resource):

    def index(self):
        return self.paginate(query=None)

    def package_filter(self, query=None):
        if query is None:
            model = self.model
        else:
            model = query

        ordering = self.request.input('ordering')

        for key in self.request.request_variables:
            if key != 'page' and key != 'ordering' and key != 'show_filters':
                model = FiltersHelper().mount(key, self.request.request_variables[key], model, ordering)

        return model

    def paginate(self, query=None):

        if query is None:
            query = self.model

        query = self.package_filter(query)

        max_rows = self.model._per_page
        current_page = None if self.request.input('page') is False else self.request.input('page')
        results = query.paginate(max_rows, 1 if current_page is None else current_page)
        total_rows_count = results.total
        max_pages = math.ceil(total_rows_count / max_rows)
        previous = None
        next = None

        if current_page is not None:
            if current_page > 1:
                if (current_page - 1) > max_pages:
                    current_page = max_pages

                previous = "{}?page={}".format(self.route_url, current_page - 1)
        else:
            current_page = 1

        if math.ceil(total_rows_count / max_rows) > current_page:
            next = "{}?page={}".format(self.route_url, current_page + 1)

        response_format = {
            "count": total_rows_count,
            "previous": previous,
            "next": next,
            "results": json.loads(results.to_json())
        }

        filters = self.request.input('show_filters')
        if filters is not False:
            response_format['filters'] = FiltersHelper().get_allowed_filters()

        return response_format



class FiltersHelper:
    filters = {
        "gt": ">",
        "gte": ">=",
        "lt": "<",
        "lte": "<=",
        "not": "!=",
        "contains": "like",
        "not_contains": "not like",
        "starts_with": "like",
        "not_starts_with": "not like",
        "ends_with": "like",
        "not_ends_with": "not like",
        "between": "between",
        "not_between": "not_between",
        "in": "in",
        "not_in": "not_in",
    }

    def mount(self, input, param, model, ordering=False):

        field = input.split('__')[0]
        try:
            ini_filter = input.split('__')[1]
        except:
            ini_filter = None

        if ini_filter is not None:
            filter = self.filters[ini_filter]
        else:
            filter = None

        if filter is not None and "like" in filter:
            param = self.mount_like(ini_filter, param)
        else:
            param = "{}".format(param) if isinstance(param, str) else param

        if filter == 'between':
            model = model.where_between(field, self.param_to_array(param))
        elif filter == 'not_between':
            model = model.where_not_between(field, self.param_to_array(param))
        elif filter == 'in':
            model = model.where_in(field, self.param_to_array(param))
        elif filter == 'not_in':
            model = model.where_not_in(field, self.param_to_array(param))
        elif filter is not None:
            model = model.where(field, filter, param)
        else:
            model = model.where(field, param)

        if ordering is not False:
            if ordering[:1] == '-':
                order_by_field = ordering[1:]
                direction = "DESC"
            else:
                order_by_field = ordering
                direction = "ASC"

            model = model.order_by(order_by_field, direction)

        return model

    def param_to_array(self, param):
        return param.split(',')

    def mount_like(self, filter, param):
        like_filters = {
            "contains": "%{}%".format(param),
            "not_contains": "%{}%".format(param),
            "starts_with": "{}%".format(param),
            "not_starts_with": "{}%".format(param),
            "ends_with": "%{}".format(param),
            "not_ends_with": "%{}".format(param),
        }

        return like_filters[filter]

    def get_allowed_filters(self):
        allowed_filters = []
        for key in self.filters:
            allowed_filters.append(key)

        return allowed_filters



from setuptools import setup

setup(
    name="masonite-api-pagination",
    author="Raphael Schubert",
    author_email="rfswdp@gmail.com",
    description="Simple yet powerful masonite-api pagination",
    url="https://github.com/rfschubert/masonite_api_pagination",
    version='0.2.1',
    packages=[
        'masonite_api_pagination'
    ],
    install_requires=[
        'masonite',
        'masonite-api'
    ],
    include_package_data=True,
)